/*
 * Fabric3
 * Copyright (c) 2009-2013 Metaform Systems
 *
 * Fabric3 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version, with the
 * following exception:
 *
 * Linking this software statically or dynamically with other
 * modules is making a combined work based on this software.
 * Thus, the terms and conditions of the GNU General Public
 * License cover the whole combination.
 *
 * As a special exception, the copyright holders of this software
 * give you permission to link this software with independent
 * modules to produce an executable, regardless of the license
 * terms of these independent modules, and to copy and distribute
 * the resulting executable under terms of your choice, provided
 * that you also meet, for each linked independent module, the
 * terms and conditions of the license of that module. An
 * independent module is a module which is not derived from or
 * based on this software. If you modify this software, you may
 * extend this exception to your version of the software, but
 * you are not obligated to do so. If you do not wish to do so,
 * delete this exception statement from your version.
 *
 * Fabric3 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the
 * GNU General Public License along with Fabric3.
 * If not, see <http://www.gnu.org/licenses/>.
*/
package org.fabric3.binding.jms.runtime.container;

import java.net.URI;

import org.fabric3.api.annotation.monitor.Debug;
import org.fabric3.api.annotation.monitor.Severe;

/**
 *
 */
public interface MessageContainerMonitor {

    @Severe("All receivers are paused, possibly as a result of rejected work for {0}")
    void pauseError(String message);

    @Severe("The number is below the minimum threshold, possibly as a result of rejected work for {0}")
    void minimumError(String message);

    @Severe("Listener threw an exception for {0}")
    void listenerError(String uri, Throwable e);

    @Severe("Error refreshing connection for for {1}")
    void connectionError(String uri, Throwable e);

    @Severe("Error stopping connection for {0}")
    void stopConnectionError(URI uri, Throwable e);

    @Severe("Error receiving message for {0}")
    void receiveError(URI uri, Throwable e);

    @Debug("Receiver scheduled: {0}")
    void scheduledReceiver(String name);

    @Debug("Number of receivers increased to {0}")
    void increaseReceivers(int count);

    @Debug("Number of receivers decreased to {0}")
    void decreaseReceivers(int count);

    @Debug("Error starting connection {0}")
    void startConnectionError(Throwable e);

    @Debug("Work has been rejected with the following exception")
    void reject(Exception e);
}
